const runtime = require("@isomorphic/runtime");
const tree = require("@isomorphic/tree");

const assets = require("./assets");
const fs = require("sf-fs");

const page = require("./transform/page");
const post = require("./transform/post");
const redirects = require("./redirects");
const getComponents = require("./components");

Error.stackTraceLimit = 10000;

module.exports = function ({ site, drafts = false, source, destination, cache })
{
    const components = getComponents({ source });
    const common = { site, components, destination, cache };

    require("child_process").execSync(`rm -rf ${cache}`);

    runtime(
        <redirects { ...common }>
            <pages source = { `${source}/pages` } { ...common }>
                <metadata>
                    <posts source = { `${source}/posts` } { ...common }>
                        <assets source = { `${source}/pages` } { ...common } />
                    </posts>
                    { drafts && <posts source = { `${source}/drafts` } { ...common }/> }
                </metadata>
            </pages>
        </redirects>);
    
    console.log("AVERAGE: " + (global.total / global.count));
}

function pages({ site, components, source, destination, cache, children })
{
    const options = { components, props: { site, posts: children } };
    const transforms = [{ match: page.match, transform: page, options }];

    return [
        <tree { ...{ source, transforms, destination, cache } }/>,
        children
    ];
}

function metadata({ children })
{
    return children
        .filter(child => child)
        .map(child => child.frontmatter || child.metadata);
}

function posts({ site, components, source, destination, cache, children })
{
    const options = { components, props: { site }, assets:children[0] || { } };
    const transforms = [{ match: `${source}/*`, transform: post, directories: true, options }];

    if (!require("fs").existsSync(source))
        return [];

    return <tree { ...{ source, transforms, destination, cache } }/>;
}
